#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import os
import unittest

import misspellings_lib as misspellings


BASE_PATH = os.path.dirname(__file__)


class Tests(unittest.TestCase):
    def test_missing_ms_list(self):
        with self.assertRaises(IOError):
            misspellings.Misspellings(
                os.path.join(BASE_PATH, 'missing_msl.txt')
            )

    def test_broken_ms_list(self):
        with self.assertRaises(ValueError):
            misspellings.Misspellings(
                os.path.join(BASE_PATH, 'broken_msl.txt')
            )

    def test_missing_file(self):
        ms = misspellings.Misspellings()
        errors, results = ms.check(os.path.join(BASE_PATH, 'missing_source.c'))
        self.assertTrue(errors)

    def test_good_file(self):
        ms = misspellings.Misspellings()
        errors, results = ms.check(
            os.path.join(BASE_PATH, 'nine_mispellings.c')
        )
        self.assertEqual(len(errors), 0)
        self.assertEqual(len(results), 9)

    def test_more_complex_file(self):
        ms = misspellings.Misspellings()
        errors, results = ms.check(
            os.path.join(BASE_PATH, 'various_spellings.c')
        )
        self.assertEqual(len(errors), 0)
        self.assertEqual(len(results), 7)


class UtilityFunctionTestCase(unittest.TestCase):
    def test_same_case(self):
        self.assertEqual(
            'Apple',
            misspellings.same_case(source='Apple', destination='apple'),
        )

        # Do not make lowercase as "Apple" may be the first word in a sentence.
        self.assertEqual(
            'Apple',
            misspellings.same_case(source='apple', destination='Apple'),
        )

    def test_same_case_with_empty_destination(self):
        self.assertEqual(
            '', misspellings.same_case(source='apple', destination='')
        )
        self.assertEqual(
            '', misspellings.same_case(source='Apple', destination='')
        )

    def test_same_case_with_empty_source(self):
        self.assertEqual(
            'apple', misspellings.same_case(source='', destination='apple')
        )
        self.assertEqual(
            'Apple', misspellings.same_case(source='', destination='Apple')
        )

    def test_split_words(self):
        self.assertEqual(
            ['one', 'two', 'three'], misspellings.split_words('one two three')
        )

    def test_split_words_with_underscores(self):
        self.assertEqual(
            ['one', 'two', 'three'], misspellings.split_words('one_two_three')
        )
        self.assertEqual(
            ['one', 'two', 'three'],
            misspellings.split_words('one__two__three'),
        )
        self.assertEqual(
            ['one', 'two', 'three', 'four'],
            misspellings.split_words('one_two_three four'),
        )

    def test_split_words_with_punctuation(self):
        self.assertEqual(['one', 'two'], misspellings.split_words('one, two'))
        self.assertEqual(
            ['a', 'sentence', ''], misspellings.split_words('a sentence.')
        )

    def test_split_words_with_numbers(self):
        self.assertEqual(
            ['upper', 'lower'], misspellings.split_words('upper2lower')
        )

    def test_split_words_with_camel_case(self):
        self.assertEqual(
            ['one', 'Two', 'Three'], misspellings.split_words('oneTwoThree')
        )
        self.assertEqual(
            ['one', 'Two', 'Three', 'Four'],
            misspellings.split_words('oneTwoThreeFour'),
        )
        self.assertEqual(
            ['one', 'Two', 'Three', 'four'],
            misspellings.split_words('oneTwoThree_four'),
        )
        self.assertEqual(
            ['one', 'Two', 'Three', 'four', 'five'],
            misspellings.split_words('oneTwoThree_four five'),
        )
        self.assertEqual(
            ['foo', 'Up', 'To', 'Bar'], misspellings.split_words('fooUpToBar')
        )

    def test_split_words_with_other_characters(self):
        self.assertEqual(
            ['the', 'big', 'cat'], misspellings.split_words('the%big$cat')
        )

    def test_normalize(self):
        self.assertEqual('alpha', misspellings.normalize('"alpha".'))


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

"""Misspellings module.

Take a list of files, check them against a list of misspelled words.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import collections
import io
import json
import os
import re
import string


__version__ = '1.0.2'


_NORM_REGEX = re.compile(r'(?<=[a-z])(?=[A-Z])')
_WORD_REGEX = re.compile(r'[\s_0-9\W]+', flags=re.UNICODE)


def normalize(word):
    """Return word with symbols stripped from its ends."""
    return word.strip(string.punctuation)


def split_words(line):
    """Return the list of words contained in a line."""
    # Normalize any camel cased words first.
    line = ' '.join(w for w in _NORM_REGEX.split(line) if w)

    return [normalize(w) for w in _WORD_REGEX.split(line)]


def same_case(source, destination):
    """Return destination with same case as source."""
    if source and source[:1].isupper():
        return destination.capitalize()
    else:
        return destination


class Misspellings(object):

    """Detects misspelled words in files."""

    def __init__(self, misspelling_file=None):
        """Initialises an Misspellings instance.

        Args:
          files: List of files to check.  More can be added with add().
          misspelling_file: Filename with a list of misspelled words
                            and their alternatives.

        Raises:
          IOError: Raised if misspelling_file can't be found.
          ValueError: Raised if misspelling_file isn't correctly formatted.

        """
        if misspelling_file:
            self._misspelling_dict = collections.defaultdict(list)
            with io.open(misspelling_file, 'r') as f:
                for line in f:
                    bad_word, correction = line.strip().split(' ', 1)
                    self._misspelling_dict[bad_word].append(correction)
        else:
            self._misspelling_dict = {}
            for dictionary in ['wikipedia.json', 'custom.json']:
                with io.open(
                    os.path.join(os.path.dirname(__file__), dictionary)
                ) as input_file:
                    self._misspelling_dict.update(json.load(input_file))

    def check(self, filename):
        """Checks the files for misspellings.

        Returns:
          (errors, results)
          errors: List of system errors, usually file access errors.
          results: List of spelling errors - each tuple is filename,
                   line number and misspelled word.

        """
        errors = []
        results = []
        if not os.path.isdir(filename):
            try:
                with io.open(filename, 'r') as f:
                    line_ct = 1
                    for line in f:
                        for word in split_words(line):
                            if (
                                word in self._misspelling_dict
                                or word.lower() in self._misspelling_dict
                            ):
                                results.append([filename, line_ct, word])
                        line_ct += 1
            except UnicodeDecodeError:
                pass
            except IOError as exception:
                errors.append(exception)
        return errors, results

    def suggestions(self, word):
        """Returns a list of suggestions for a misspelled word.

        Args:
          word: The word to check.

        Returns:
          List of zero or more suggested replacements for word.

        """
        suggestions = set(self._misspelling_dict.get(word, [])).union(
            set(self._misspelling_dict.get(word.lower(), []))
        )
        return sorted(
            same_case(source=word, destination=w) for w in suggestions
        )

    def dump_misspelling_list(self):
        """Returns a list of misspelled words and corrections."""
        results = []
        for bad_word in sorted(self._misspelling_dict.keys()):
            for correction in self._misspelling_dict[bad_word]:
                results.append([bad_word, correction])
        return results



#!/usr/bin/env python

from __future__ import unicode_literals

import os
import subprocess
import unittest


BASE_PATH = os.path.abspath(os.path.dirname(__file__))
CLI = os.path.join(BASE_PATH, os.pardir, 'misspellings')


class Tests(unittest.TestCase):

    """Test the CLI.

    USAGE: misspellings [-f file] [files]
    Checks files for common spelling mistakes.
      -f file: File containing list of files to check.
      -m file: File containing list of misspelled words & corrections.
      -d     : Dump the list of misspelled words.
      -s file: Create a shell script to interactively correct the file.
      files: Zero or more files to check.

    """

    def test_good_file(self):
        p = subprocess.Popen(
            [CLI, 'nine_mispellings.c'],
            cwd=BASE_PATH,
            stderr=subprocess.PIPE,
            stdout=subprocess.PIPE,
        )
        (output, error_output) = p.communicate()
        self.assertEqual(error_output.decode(), '')
        self.assertEqual(len(output.decode().split('\n')), 10)
        self.assertEqual(p.returncode, 2)

    def test_bad_file(self):
        p = subprocess.Popen(
            [CLI, 'missing.c'],
            cwd=BASE_PATH,
            stderr=subprocess.PIPE,
            stdout=subprocess.PIPE,
        )
        (output, error_output) = p.communicate()
        self.assertEqual(output.decode(), '')
        self.assertEqual(len(error_output.decode().split('\n')), 2)
        self.assertEqual(p.returncode, 0)

    def test_good_flag_f(self):
        p = subprocess.Popen(
            [CLI, '-f', 'good_file_list'],
            cwd=BASE_PATH,
            stderr=subprocess.PIPE,
            stdout=subprocess.PIPE,
        )
        (output, error_output) = p.communicate()
        self.assertEqual(error_output.decode(), '')
        self.assertEqual(len(output.decode().split('\n')), 10)
        self.assertEqual(p.returncode, 2)

    def test_bad_flag_f(self):
        p = subprocess.Popen(
            [CLI, '-f', 'broken_file_list'],
            cwd=BASE_PATH,
            stderr=subprocess.PIPE,
            stdout=subprocess.PIPE,
        )
        (output, error_output) = p.communicate()
        self.assertEqual(output.decode(), '')
        self.assertEqual(len(error_output.decode().split('\n')), 2)
        self.assertEqual(p.returncode, 0)

    def test_bad_flag_m(self):
        p = subprocess.Popen(
            [CLI, '-d', '-m', 'broken_msl.txt'],
            cwd=BASE_PATH,
            stderr=subprocess.PIPE,
            stdout=subprocess.PIPE,
        )
        (output, error_output) = p.communicate()
        self.assertIn('ValueError', error_output.decode())
        self.assertEqual(output.decode(), '')
        self.assertEqual(p.returncode, 1)

    def test_good_flag_m(self):
        p = subprocess.Popen(
            [CLI, '-d', '-m', 'small_msl.txt'],
            cwd=BASE_PATH,
            stderr=subprocess.PIPE,
            stdout=subprocess.PIPE,
        )
        (output, error_output) = p.communicate()
        self.assertEqual(error_output.decode(), '')
        self.assertEqual(len(output.decode().split('\n')), 3)
        self.assertEqual(p.returncode, 0)

    def test_bad_flag_s(self):
        p = subprocess.Popen(
            [CLI, '-s', 'various_spellings.c'],
            cwd=BASE_PATH,
            stderr=subprocess.PIPE,
            stdout=subprocess.PIPE,
        )
        (output, error_output) = p.communicate()
        self.assertIn('must not exist', error_output.decode())
        self.assertEqual(output.decode(), '')
        self.assertEqual(p.returncode, 2)

    def test_good_flag_s(self):
        test_out = os.path.join(BASE_PATH, 'various_spellings.test_out')
        good_out = os.path.join(BASE_PATH, 'various_spellings.good_out')
        if os.path.exists(test_out):
            os.unlink(test_out)
        p = subprocess.Popen(
            [CLI, '-s', test_out, 'various_spellings.c'],
            cwd=BASE_PATH,
            stderr=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stdin=subprocess.PIPE,
        )
        (output, error_output) = p.communicate(input='\n'.encode('utf8'))
        self.assertEqual(error_output.decode(), '')
        self.assertIn('withdrawl', output.decode())
        self.assertEqual(p.returncode, 0)

        with open(good_out, 'r') as good_file:
            good_contents = good_file.readlines()
        with open(test_out, 'r') as test_file:
            test_contents = test_file.readlines()
        self.assertListEqual(test_contents, good_contents)

    def test_standard_in(self):
        p = subprocess.Popen(
            [CLI, '-f', '-'],
            cwd=BASE_PATH,
            stderr=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stdin=subprocess.PIPE,
        )
        (output, error_output) = p.communicate(
            input='nine_mispellings.c\n'.encode('utf8')
        )
        self.assertEqual(error_output.decode(), '')
        self.assertEqual(len(output.decode().split('\n')), 10)
        self.assertEqual(p.returncode, 2)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

"""Installer for misspelling-check.

This installs the misspelling command and the misspellings.check module.
"""

import ast
import sys
from distutils.core import Command
from distutils.core import setup
import unittest


def version():
    """Return version string."""
    with open('misspellings_lib/__init__.py') as input_file:
        for line in input_file:
            if line.startswith('__version__'):
                return ast.parse(line).body[0].value.s


class TestCommand(Command):
    description = 'Runs all available tests.'
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        tests = unittest.TestLoader().discover('tests')
        runner = unittest.TextTestRunner(verbosity=2)
        result = runner.run(tests)
        if not result.wasSuccessful():
            sys.exit(1)


with open('README.rst') as readme:
    setup(
        cmdclass={'test': TestCommand},
        name='misspellings_lib',
        version=version(),
        url='https://github.com/mazulo/misspellings',
        download_url='https://github.com/mazulo/misspellings/tarball/%s'
        % version(),
        author='Patrick Mazulo',
        author_email='pmazulo@gmail.com',
        description='A tool to detect misspellings with opinionated additions',
        long_description=readme.read(),
        packages=['misspellings_lib'],
        package_data={'misspellings_lib': ['custom.json', 'wikipedia.json']},
        scripts=[
            'misspellings',
        ],
        keywords='check, code, spelling, spellcheck',
        license='GNU General Public License v3',
        platforms=['POSIX'],
        classifiers=[
            'Development Status :: 5 - Production/Stable',
            'Environment :: Console',
            'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
            'Operating System :: OS Independent',
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3',
            'Topic :: Utilities',
        ],
    )
